/* Copyright 2023
     Carlos Durán Domínguez <wurt@wurtshell.com>.

   This file is part of Guile Gettext.

   Guile is free software: you can redistribute it and/or modify it
   under the terms of the GNU Lesser General Public License as published
   by the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Guile is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
   License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with Guile Gettext.  If not, see
   <https://www.gnu.org/licenses/>.  */


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <locale.h>

#include <libguile/feature.h>
#include <libguile/gsubr.h>
#include <libguile/numbers.h>
#include <libguile/strings.h>

#include "libgettext.h"

#include "gettext.h"

int
scm_i_to_lc_category (SCM category, int allow_lc_all)
{
  int c_category = scm_to_int (category);
  switch (c_category)
    {
#ifdef LC_CTYPE
    case LC_CTYPE:
#endif
#ifdef LC_NUMERIC
    case LC_NUMERIC:
#endif
#ifdef LC_COLLATE
    case LC_COLLATE:
#endif
#ifdef LC_TIME
    case LC_TIME:
#endif
#ifdef LC_MONETARY
    case LC_MONETARY:
#endif
#ifdef LC_MESSAGES
    case LC_MESSAGES:
#endif
#ifdef LC_PAPER
    case LC_PAPER:
#endif
#ifdef LC_NAME
    case LC_NAME:
#endif
#ifdef LC_ADDRESS
    case LC_ADDRESS:
#endif
#ifdef LC_TELEPHONE
    case LC_TELEPHONE:
#endif
#ifdef LC_MEASUREMENT
    case LC_MEASUREMENT:
#endif
#ifdef LC_IDENTIFICATION
    case LC_IDENTIFICATION:
#endif
      return c_category;
#ifdef LC_ALL
    case LC_ALL:
      if (allow_lc_all)
	return c_category;
#endif
    }
  scm_wrong_type_arg (0, 0, category);
}

SCM_DEFINE (scm_pgettext, "pgettext", 2, 2, 0,
	    (SCM msgctxt, SCM msgid, SCM domain, SCM category),
	    "Return the translation of @var{msgid} in the message domain "
	    "@var{domain} restricted to the context given by MSGCTXT. "
	    "@var{domain} is optional and defaults to the domain set through "
	    "(textdomain).  @var{category} is optional and defaults to "
	    "LC_MESSAGES.")
#define FUNC_NAME s_scm_pgettext
{
  char *c_msgctxt;
  char *c_msgid;
  char const *c_result;
  SCM result;

  scm_dynwind_begin (0);

  c_msgctxt = scm_to_locale_string (msgctxt);
  scm_dynwind_free (c_msgctxt);

  c_msgid = scm_to_locale_string (msgid);
  scm_dynwind_free (c_msgid);

  if (SCM_UNBNDP (domain))
    {
      /* 1 argument case.  */
      c_result = pgettext_expr (c_msgctxt, c_msgid);
    }
  else
    {
      char *c_domain;

      c_domain = scm_to_locale_string (domain);
      scm_dynwind_free (c_domain);

      if (SCM_UNBNDP (category))
	{
	  /* 2 argument case.  */
	  c_result = dpgettext_expr (c_domain, c_msgctxt, c_msgid);
	}
      else
	{
	  /* 3 argument case.  */
	  int c_category;

	  c_category = scm_i_to_lc_category (category, 0);
	  c_result = dcpgettext_expr (c_domain, c_msgctxt, c_msgid, c_category);
	}
    }

  if (c_result == c_msgid)
    result = msgid;
  else
    result = scm_from_locale_string (c_result);

  scm_dynwind_end ();
  return result;
}
#undef FUNC_NAME


SCM_DEFINE (scm_npgettext, "npgettext", 4, 2, 0,
	    (SCM msgctxt, SCM msgid, SCM msgid_plural, SCM n, SCM domain,
             SCM category),
	    "Return the translation of @var{msgid}/@var{msgid_plural} in the "
	    "message domain @var{domain} restricted to the context given by "
            "MSGCTXT, with the plural form being chosen appropriately for the "
	    "number @var{n}.  @var{domain} is optional and defaults to the "
	    "domain set through (textdomain). @var{category} is optional and "
	    "defaults to LC_MESSAGES.")
#define FUNC_NAME s_scm_npgettext
{
  char *c_msgctxt;
  char *c_msgid;
  char *c_msgid_plural;
  unsigned long c_n;
  const char *c_result;
  SCM result;

  scm_dynwind_begin (0);

  c_msgctxt = scm_to_locale_string (msgctxt);
  scm_dynwind_free (c_msgctxt);

  c_msgid = scm_to_locale_string (msgid);
  scm_dynwind_free (c_msgid);

  c_msgid_plural = scm_to_locale_string (msgid_plural);
  scm_dynwind_free (c_msgid_plural);

  c_n = scm_to_ulong (n);

  if (SCM_UNBNDP (domain))
    {
      /* 3 argument case.  */
      c_result = npgettext_expr (c_msgctxt, c_msgid, c_msgid_plural, c_n);
    }
  else
    {
      char *c_domain;

      c_domain = scm_to_locale_string (domain);
      scm_dynwind_free (c_domain);

      if (SCM_UNBNDP (category))
	{
	  /* 4 argument case.  */
	  c_result = dnpgettext_expr (c_domain, c_msgctxt, c_msgid,
                                      c_msgid_plural, c_n);
	}
      else
	{
	  /* 5 argument case.  */
	  int c_category;

	  c_category = scm_i_to_lc_category (category, 0);
	  c_result = dcnpgettext_expr (c_domain, c_msgctxt, c_msgid,
                                       c_msgid_plural, c_n, c_category);
	}
    }

  if (c_result == c_msgid)
    result = msgid;
  else if (c_result == c_msgid_plural)
    result = msgid_plural;
  else
    result = scm_from_locale_string (c_result);

  scm_dynwind_end ();
  return result;
}
#undef FUNC_NAME

void
init_gettext ()
{
  #include "gettext.x"
}
